<?php

namespace SM\Service;

use SM\Factory\Sm00006001TabelaPrecoServicoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use SM\Util\Procedure\FctSincronizarVariacaoPreco;

class Sm00006001TabelaPrecoServicoService
{
    protected $connection;
    protected $tabelaPrecoServicosFactory;
    protected $environment;
    protected $newCode;

    public function __construct(Sm00006001TabelaPrecoServicoFactory $tabelaPrecoServicosFactory, Environment $environment, NewCode $newCode, EntityManager $entityManager)
    {
        $this->tabelaPrecoServicosFactory   = $tabelaPrecoServicosFactory;
        $this->environment                  = $environment;
        $this->newCode                      = $newCode;
        $this->connection                   = $entityManager->getConnection();
    }

    public function getTabelaPrecoServicoGrid()
    {
        return $this->tabelaPrecoServicosFactory->getTabelaPrecoServicoGrid();
    }

    public function getServicosTabelaPreco($parametro)
    {
        return $this->tabelaPrecoServicosFactory->getServicosTabelaPreco($parametro);
    }

    public function salvarTabelaPreco($parametros)
    {
        $cdoperador = $this->environment->getCdOperador();
        $nrorg = $this->environment->getNrOrg();
        $this->connection->beginTransaction();
        try {
            if ($parametros["CDPRECBASESERV"] == null) {
                $cdprecbaseserv = $this->newCode->getCodeByTableName("CDPRECBASESERV", $nrorg, 1, 10, true);
                $dataParams = array(
                    "CDPRECBASESERV"        => $cdprecbaseserv,
                    "DSPRECBASESERV"         => $parametros['DSPRECBASESERV'],
                    "NRORG"                  => $nrorg,
                    "NRORGINCLUSAO"          => $nrorg,
                    "CDOPERINCLUSAO"         => $cdoperador,
                    "IDATIVO"                => "S",
                    "NRMOEDA"                => $parametros['NRMOEDA'],
                );
                $this->tabelaPrecoServicosFactory->insertTabelaPreco($dataParams);
            } else {
                $dataParams = array(
                    "CDPRECBASESERV"         => $parametros['CDPRECBASESERV'],
                    "DSPRECBASESERV"         => $parametros['DSPRECBASESERV'],
                    "NRORG"                  => $nrorg,
                    "NRORGULTATU"            => $nrorg,
                    "CDOPERULTATU"           => $cdoperador,
                    "IDATIVO"                => "S",
                    "NRMOEDA"                => $parametros['NRMOEDA'],
                );
                $this->tabelaPrecoServicosFactory->updateTabelaPreco($dataParams);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteTabelaPrecoServico($parametros)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($parametros as $tabelas) {
                $dataParam = array( "CDPRECBASESERV" => $tabelas["CDPRECBASESERV"],);
                $this->tabelaPrecoServicosFactory->deleteTabelaPrecoServico($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function copiaOutraMoeda($parametros)
    {
        $cdoperador = $this->environment->getCdOperador();
        $nrorg = $this->environment->getNrOrg();
        $this->connection->beginTransaction();
        try {
            foreach ($parametros["row"] as $row) {
                $pkcdprecbaseserv = $this->newCode->getCodeByTableName("CDPRECBASESERV", $nrorg, 1, 10, true);
                $nomeTabelaPrec = $row["DSPRECBASESERV"].' - '.$parametros["NMMOEDA"];

                $params = array(
                    "CDPRECBASESERV" => $pkcdprecbaseserv,
                    "DSPRECBASESERV" => $nomeTabelaPrec,
                    "NRMOEDA"        => $parametros["NRMOEDA"],
                    "NRORG"          => $nrorg,
                    "CDOPERINCLUSAO" => $cdoperador
                );
                $this->tabelaPrecoServicosFactory->insertTabPrecos($params);
                $paramsAux = array(
                    "CDPRECBASESERV" => $row["CDPRECBASESERV"],
                );
                $dataServico = $this->tabelaPrecoServicosFactory->getDadosServico($paramsAux);
                foreach ($dataServico["data"] as $itPrecBaseServ) {
                    if ($parametros["CONVERSAO"] > 0) {
                        $calculovrpreco = $itPrecBaseServ["VRPRECO"]*$parametros["CONVERSAO"];
                    } else {
                        $calculovrpreco = abs($itPrecBaseServ["VRPRECO"]/$parametros["CONVERSAO"]);
                        $calculovrpreco = round($calculovrpreco, 2);
                    }

                    $pknrseqitprecbserv = $this->newCode->getCodeByTableName("FCT_ITPRECBASESERV", $nrorg, 1, 10, true);
                    $paramsServico = array(
                        "NRSEQITPRECBSERV" => $pknrseqitprecbserv,
                        "CDPRECBASESERV"   => $pkcdprecbaseserv,
                        "DTINIPBSERV"      => $itPrecBaseServ["DTINIPBSERV"],
                        "CDTPMEDIDASERV"   => $itPrecBaseServ["CDTPMEDIDASERV"],
                        "CDSERVORCA"       => $itPrecBaseServ["CDSERVORCA"],
                        "VRPRECO"          => $calculovrpreco,
                        "DTFIMVALPBSERV"   => $itPrecBaseServ["DTFIMVALPBSERV"],
                        "CDFILIAL"         => $itPrecBaseServ["CDFILIAL"],
                        "NRORG"            => $nrorg,
                        "DTINCLUSAO"       => $itPrecBaseServ["DTINCLUSAO"],
                        "DTULTATU"         => $itPrecBaseServ["DTULTATU"],
                        "NRORGINCLUSAO"    => $itPrecBaseServ["NRORGINCLUSAO"],
                        "CDOPERINCLUSAO"   => $cdoperador,
                        "NRORGULTATU"      => $itPrecBaseServ["NRORGULTATU"],
                        "CDOPERULTATU"     => $itPrecBaseServ["CDOPERULTATU"],
                        "IDATIVO"          => $itPrecBaseServ["IDATIVO"],
                    );
                    $this->tabelaPrecoServicosFactory->insertItPrecBaseServ($paramsServico);

                    $paramsAuxVariacao = array(
                        "NRSEQITPRECBSERV" => $itPrecBaseServ["NRSEQITPRECBSERV"],
                    );
                    $dataVariacao = $this->tabelaPrecoServicosFactory->getDadosVariacao($paramsAuxVariacao);

                    foreach ($dataVariacao["data"] as $variacao) {
                        if ($parametros["CONVERSAO"] > 0) {
                            $calculoprecovariavel = $variacao["VRPRECOVARIACAO"]*$parametros["CONVERSAO"];
                        } else {
                            $calculoprecovariavel = abs($variacao["VRPRECOVARIACAO"]/$parametros["CONVERSAO"]);
                            $calculoprecovariavel = round($calculoprecovariavel, 2);
                        }
                        $pknrvariacaoprecbaseserv = $this->newCode->getCodeByTableName("FCT_VARIACAOPRECBASESERV", $nrorg, 1, 10, true);
                        $paramsVariacao = array(
                            "NRVARIACAOPRECBASESERV"    => $pknrvariacaoprecbaseserv,
                            "NRSEQITPRECBSERV"          => $pknrseqitprecbserv,
                            "CDTPMEDIDASERV"            => $variacao["CDTPMEDIDASERV"],
                            "DTINIPBSERV"               => $variacao["DTINIPBSERV"],
                            "QTMEDIDAINIVARIACAO"       => $variacao["QTMEDIDAINIVARIACAO"],
                            "QTMEDIDAFIMVARIACAO"       => $variacao["QTMEDIDAFIMVARIACAO"],
                            "VRPRECOVARIACAO"           => $calculoprecovariavel,
                            "CDTPMEDIDAVARIAVEL"        => $variacao["CDTPMEDIDAVARIAVEL"],
                            "IDMULTIPVARIACAO"          => $variacao["IDMULTIPVARIACAO"],
                            "NRORG"                     => $nrorg,
                            "IDATIVO"                   => $variacao["IDATIVO"],
                            "DTINCLUSAO"                => $variacao["DTINCLUSAO"],
                            "DTULTATU"                  => $variacao["DTULTATU"],
                            "QTFRANQUIA"                => $variacao["QTFRANQUIA"],
                            "NRORGINCLUSAO"             => $variacao["NRORGINCLUSAO"],
                            "CDOPERINCLUSAO"            => $cdoperador,
                            "NRORGULTATU"               => $variacao["NRORGULTATU"],
                            "CDOPERULTATU"              => $variacao["CDOPERULTATU"],
                        );
                        $this->tabelaPrecoServicosFactory->insertVariacao($paramsVariacao);
                    }
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function validaServicoTabela($parametros)
    {
        $params = array(
            "CDSERVORCA"        => implode(',', $parametros["servOrca"]),
            "CDPRECBASESERV"    => $parametros["precBaseServ"],
            "CDTPMEDIDASERV"    => $parametros["tipoMedida"],
            "NRORG"             => $this->environment->getNrOrg()
        );
        $return = $this->tabelaPrecoServicosFactory->validaServicoTabela($params);
        return $return["data"][0]["VALIDASERV"] != 0;
    }

    public function salvarServicoTabela($parametros)
    {
        $cdoperador = $this->environment->getCdOperador();
        $nrorg = $this->environment->getNrOrg();
        $this->connection->beginTransaction();
        $ItPrecBServ = array();
        try {
            foreach ($parametros as $params) {
                if ($params["NRSEQITPRECBSERV"] == null) {
                    foreach ($params["CDSERVORCA"] as $cdservorca) {
                            $nrseqitprecbserv = $this->newCode->getCodeByTableName("FCT_ITPRECBASESERV", $nrorg, 1, 10, true);
                            $dataParams = array(
                                "NRSEQITPRECBSERV"       => $nrseqitprecbserv,
                                "CDPRECBASESERV"         => $params['CDPRECBASESERV'],
                                "DTINIPBSERV"            => $params['DTINIPBSERV'],
                                "CDTPMEDIDASERV"         => $params['CDTPMEDIDASERV'],
                                "CDSERVORCA"             => $cdservorca,
                                "VRPRECO"                => $params['VRPRECO'],
                                "DTFIMVALPBSERV"         => $params['DTFIMVALPBSERV'],
                                "NRORG"                  => $nrorg,
                                "NRORGINCLUSAO"          => $nrorg,
                                "CDOPERINCLUSAO"         => $cdoperador,
                                "IDATIVO"                => "S"
                            );
                            $this->tabelaPrecoServicosFactory->insertServicoTabelaPreco($dataParams);
                            $ItPrecBServ[] = $nrseqitprecbserv;
                    }
                } else {
                    $dataParams = array(
                        "NRSEQITPRECBSERV"       => $params['NRSEQITPRECBSERV'],
                        "CDPRECBASESERV"         => $params['CDPRECBASESERV'],
                        "DTINIPBSERV"            => $params['DTINIPBSERV'],
                        "CDTPMEDIDASERV"         => $params['CDTPMEDIDASERV'],
                        "CDSERVORCA"             => $params['CDSERVORCA'],
                        "VRPRECO"                => $params['VRPRECO'],
                        "DTFIMVALPBSERV"         => $params['DTFIMVALPBSERV'],
                        "NRORG"                  => $nrorg,
                        "NRORGULTATU"          => $nrorg,
                        "CDOPERULTATU"         => $cdoperador,
                        "IDATIVO"                => "S"
                    );
                    $this->tabelaPrecoServicosFactory->updateServicoTabelaPreco($dataParams);
                }
            }
            $this->connection->commit();
            return $ItPrecBServ;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function executarProcedureFctSincronizarVariacaoPreco($params)
    {
        $cdoperador = $this->environment->getCdOperador();
        $nrorg = $this->environment->getNrOrg();
        $this->connection->beginTransaction();
        try {
            foreach ($params["NRSEQITPRECBSERV"] as $itPrecBaseServ) {
                $procedure = new FctSincronizarVariacaoPreco($this->connection);
                $procedure->executar($itPrecBaseServ, $params["TIPO"], $nrorg, $cdoperador);
            }
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteServicosTabelaPrecoServico($parametros)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($parametros as $tabelas) {
                $dataParam = array( "NRSEQITPRECBSERV" => $tabelas["NRSEQITPRECBSERV"],);
                $this->tabelaPrecoServicosFactory->deleteServicosTabelaPrecoServico($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getMedidaVariavelTabelaPreco($parametro)
    {
        return $this->tabelaPrecoServicosFactory->getMedidaVariavelTabelaPreco($parametro);
    }

    public function validaFranquiaFaixa($parametros)
    {
        $dataParams = array(
            "NRSEQITPRECBSERV"   => $parametros["NRSEQITPRECBSERV"],
            "CDTPMEDIDASERV"     => $parametros["CDTPMEDIDASERV"],
            "CDTPMEDIDAVARIAVEL" => $parametros["CDTPMEDIDAVARIAVEL"],
            "QTFRANQUIA"         => $parametros["QTFRANQUIA"],
            "NRORG"              => $this->environment->getNrOrg()
        );
        
        return $this->tabelaPrecoServicosFactory->validaFranquiaFaixa($dataParams);
    }

    public function insereVariacaoPreco($parametros)
    {
        $nrorg = $this->environment->getNrOrg();
        $operador =$this->environment->getCdOperador();
        if ($parametros["SINCRONIZAR"] == 'S') {
            $paramsSinc = array(
                    "CDTPMEDIDASERV"          => $parametros["CDTPMEDIDASERV"],
                    "CDTPMEDIDAVARIAVEL"      => $parametros["CDTPMEDIDAVARIAVEL"],
                    "NRORG"                   => $nrorg,
            );
            $faixas = $this->tabelaPrecoServicosFactory->sinconizaMedidaVariavel($paramsSinc);
            if (count($faixas["data"]) == 0) {
                $this->geraVarPadrao($parametros["NRSEQITPRECBSERV"], $parametros["CDTPMEDIDASERV"], $parametros["CDTPMEDIDAVARIAVEL"], $parametros["VRPRECOVARIACAO"], $parametros["IDMULTIPVARIACAO"], $parametros["QTFRANQUIA"], $nrorg, $operador, $parametros["USAFRANQUIA"]);
            } else {
                $countFranquia = 0;
                foreach ($faixas["data"] as $faixa) {
                    $qtIniVariacao = $faixa['QTMEDIDAINIVARIACAO'];
                    $qtFimVariacao = $faixa['QTMEDIDAFIMVARIACAO'];
                    $percentual    = $faixa['VRPERCVARIACAO'];
                    if ($parametros["USAFRANQUIA"] == 'S') {
                        $this->geraFaixasFranquia($parametros["NRSEQITPRECBSERV"], $parametros["CDTPMEDIDASERV"], $parametros["CDTPMEDIDAVARIAVEL"], $qtIniVariacao, $qtFimVariacao, $parametros["VRPRECOVARIACAO"], $percentual, $parametros["IDMULTIPVARIACAO"], $parametros["QTFRANQUIA"], $nrorg, $operador, $countFranquia);
                        $countFranquia = $countFranquia+1;
                    } else {
                        $this->geraFaixas($parametros["NRSEQITPRECBSERV"], $parametros["CDTPMEDIDASERV"], $parametros["CDTPMEDIDAVARIAVEL"], $qtIniVariacao, $qtFimVariacao, $parametros["VRPRECOVARIACAO"], $percentual, $parametros["IDMULTIPVARIACAO"], $parametros["QTFRANQUIA"], $nrorg, $operador);
                    }
                }
            }
        } else {
            $this->geraVarPadrao($parametros["NRSEQITPRECBSERV"], $parametros["CDTPMEDIDASERV"], $parametros["CDTPMEDIDAVARIAVEL"], $parametros["VRPRECOVARIACAO"], $parametros["IDMULTIPVARIACAO"], $parametros["QTFRANQUIA"], $nrorg, $operador, $parametros["USAFRANQUIA"]);
        }
    }

    public function geraVarPadrao($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $vrPrecoVariacao, $idMultipVariacao, $qtFranquia, $nrorg, $operador, $usaFranquia)
    {
        $this->connection->beginTransaction();
        try {
            if ($usaFranquia == 'S') {
                $qtmedidafim = $qtFranquia;
                $vrPreco = 0;
            } else {
                $qtmedidafim = 99999999;
                $vrPreco = $vrPrecoVariacao;
            };
            $novoCodigo = $this->newCode->getCodeByTableName("FCT_VARIACAOPRECBASESERV", $nrorg, 1, 10, true);
            $params = array(
                    "NRVARIACAOPRECBASESERV"  => $novoCodigo,
                    "NRSEQITPRECBSERV"        => $nrSeqItPrecBServ,
                    "CDTPMEDIDASERV"          => $cdTpMedidaServ,
                    "CDTPMEDIDAVARIAVEL"      => $cdtpMedidaVariavel,
                    "QTMEDIDAINIVARIACAO"     => 0,
                    "QTMEDIDAFIMVARIACAO"     => $qtmedidafim,
                    "VRPRECOVARIACAO"         => $vrPreco,
                    "IDMULTIPVARIACAO"        => $idMultipVariacao,
                    "QTFRANQUIA"              => $qtFranquia,
                    "NRORG"                   => $nrorg,
                    "OPERADOR"                => $operador,
            );
            $this->tabelaPrecoServicosFactory->insereVariacaoPreco($params);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function geraFaixasFranquia($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacao, $qtFimVariacao, $vrPrecoVariacao, $percentual, $idMultipVariacao, $qtFranquia, $nrorg, $operador, $countFranquia)
    {
        if ($countFranquia == 0) {
            $this->geraVarPadrao($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, 0, $idMultipVariacao, $qtFranquia, $nrorg, $operador, 'S');
        }
        if (($qtIniVariacao <= $qtFranquia) && ($qtFimVariacao > $qtFranquia)) {
            $qtIniVariacaoAux =  $qtFranquia+0.01;
            $this->geraFaixas($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacaoAux, $qtFimVariacao, $vrPrecoVariacao, $percentual, $idMultipVariacao, $qtFranquia, $nrorg, $operador);
        } elseif (($qtIniVariacao > $qtFranquia) && ($qtFimVariacao > $qtFranquia)) {
            $this->geraFaixas($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacao, $qtFimVariacao, $vrPrecoVariacao, $percentual, $idMultipVariacao, $qtFranquia, $nrorg, $operador);
        }
    }

    public function geraFaixas($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacao, $qtFimVariacao, $vrPrecoVariacao, $percentual, $idMultipVariacao, $qtFranquia, $nrorg, $operador)
    {
        if ($percentual > 0) {
            $precovariacaocalc = ($vrPrecoVariacao*$percentual)/100;
            $precovariacao     = $vrPrecoVariacao - $precovariacaocalc;
        }
        if ($percentual < 0) {
            $precovariacaocalc = $percentual* -1;
            $precovariacaocalc = ($vrPrecoVariacao*$precovariacaocalc)/100;
            $precovariacao     = $vrPrecoVariacao + $precovariacaocalc;
        }
        if ($percentual == 0) {
             $precovariacao = $vrPrecoVariacao;
        }
        $this->geraVariacao($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacao, $qtFimVariacao, $precovariacao, $idMultipVariacao, $qtFranquia, $nrorg, $operador);
    }

    public function geraVariacao($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacao, $qtFimVariacao, $vrPrecoVariacao, $idMultipVariacao, $qtFranquia, $nrorg, $operador)
    {
        $this->connection->beginTransaction();
        try {
            $novoCodigo = $this->newCode->getCodeByTableName("FCT_VARIACAOPRECBASESERV", $nrorg, 1, 10, true);
            $params = array(
                "NRVARIACAOPRECBASESERV"  => $novoCodigo,
                "NRSEQITPRECBSERV"        => $nrSeqItPrecBServ,
                "CDTPMEDIDASERV"          => $cdTpMedidaServ,
                "CDTPMEDIDAVARIAVEL"      => $cdtpMedidaVariavel,
                "QTMEDIDAINIVARIACAO"     => $qtIniVariacao,
                "QTMEDIDAFIMVARIACAO"     => $qtFimVariacao,
                "VRPRECOVARIACAO"         => $vrPrecoVariacao,
                "IDMULTIPVARIACAO"        => $idMultipVariacao,
                "QTFRANQUIA"              => $qtFranquia,
                "NRORG"                   => $nrorg,
                "OPERADOR"                => $operador
            );
            $this->tabelaPrecoServicosFactory->insereVariacaoPreco($params);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function updateVariacao($parametros)
    {
        $nrorg = $this->environment->getNrOrg();
        $operador = $this->environment->getCdOperador();
        $this->connection->beginTransaction();
        try {
            foreach ($parametros["editedrows"] as $params) {
                $dataParams = array(
                    "IDMULTIPVARIACAO"       => $params["IDMULTIPVARIACAO"],
                    "OPERADOR"               => $operador,
                    "NRORG"                  => $nrorg,
                    "NRSEQITPRECBSERV"       => $parametros["NRSEQITPRECBSERV"],
                    "CDTPMEDIDASERV"         => $parametros["CDTPMEDIDASERV"],
                    "CDTPMEDIDAVARIAVEL"     => $params["CDTPMEDIDAVARIAVEL"],
                    "QTFRANQUIA"             => $params["QTFRANQUIA"],
                    "NRVARIACAOPRECBASESERV" => $params["NRVARIACAOPRECBASESERV"],
                );
                $this->tabelaPrecoServicosFactory->updateVariacao($dataParams);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteServicosTabelaPrecoMedidaVariavel($parametros)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($parametros as $tabelas) {
                $dataParam = array( "NRVARIACAOPRECBASESERV" => $tabelas["NRVARIACAOPRECBASESERV"],);
                $this->tabelaPrecoServicosFactory->deleteServicosTabelaPrecoMedidaVariavel($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getVariacaoPrecoTabelaPreco($parametro)
    {
        return $this->tabelaPrecoServicosFactory->getVariacaoPrecoTabelaPreco($parametro);
    }

    public function validaTipoMedidaVariavel($parametros)
    {
        $params = array (
            "NRSEQITPRECBSERV"       => $parametros["NRSEQITPRECBSERV"],
            "CDTPMEDIDASERV"         => $parametros["CDTPMEDIDASERV"],
            "CDTPMEDIDAVARIAVEL"     => $parametros["CDTPMEDIDAVARIAVEL"],
            "NRVARIACAOPRECBASESERV" => $parametros["NRVARIACAOPRECBASESERV"],
            "NRORG"                  => $this->environment->getNrOrg(),
        );
        
        return $this->tabelaPrecoServicosFactory->validaTipoMedidaVariavel($params);
    }

    public function salvarVariacaoTipoDeMedida($parametros)
    {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();
        $this->connection->beginTransaction();
        try {
            $novoCodigo = $this->newCode->getCodeByTableName("FCT_VARIACAOPRECBASESERV", $nrorg, 1, 10, true);
            $dataParams = array(
                "NRVARIACAOPRECBASESERV"    => $novoCodigo,
                "NRSEQITPRECBSERV"          => $parametros["NRSEQITPRECBSERV"],
                "CDTPMEDIDASERV"            => $parametros["CDTPMEDIDASERV"],
                "CDTPMEDIDAVARIAVEL"        => $parametros["CDTPMEDIDAVARIAVEL"],
                "IDMULTIPVARIACAO"          => $parametros["IDMULTIPVARIACAO"],
                "QTFRANQUIA"                => $parametros["QTFRANQUIA"],
                "QTMEDIDAINIVARIACAO"       => $parametros["QTMEDIDAINIVARIACAO"],
                "QTMEDIDAFIMVARIACAO"       => $parametros["QTMEDIDAFIMVARIACAO"],
                "VRPRECOVARIACAO"           => $parametros["VRPRECOVARIACAO"],
                "DTINIPBSERV"               => $parametros["DTINIPBSERV"],
                "DTFIMVALPBSERV"            => $parametros["DTFIMVALPBSERV"],
                "NRORG"                     => $nrorg,
                "OPERADOR"                  => $cdoperador,
                "IDATIVO"                   => 'S'
            );
            $this->tabelaPrecoServicosFactory->insereVariacaoPreco($dataParams);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
