import {
    Http, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';
import {
    AlertService, Button, DialogService, Form, LoadingService, Modal, ModalService, Select, SelectMultiple, Tabs, Text, TextInput, Textarea,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class AgrupamentoContratoController {
    private modalAdicionar!: Modal;

    public modalContratos!: Modal;

    public visibilidade: boolean = true;

    public currentRow: any = {};

    get visibilidadeDoFrame() {
        return !this.visibilidade;
    }

    public visualizarDadosGerais({ row }: any) {
        this.visibilidade = false;
        this.currentRow = row;
        this.recarregarTabs();
    }

    public voltarPaginaInicial() {
        this.visibilidade = true;
    }

    public expandir({ component }: any) {
        const form = Metadata.getInstance<Form>(component.name.split('_')[1]);
        form.isVisible = !form.isVisible;

        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }

    public alterarLarguraMaxima({ component }: any) {
        component.getColumn('NMAGRUPAMENTO').maxWidth = '450px';
        component.getColumn('NMAGRUPAMENTO').overflow = 'wrap';
        this.dispararFiltro({ component });
    }

    public async abrirModalAdicionarAgrupamento() {
        this.modalAdicionar = await ModalService.createFromJson('Faturamento/Modal/sm_next_00049_001_modal_adicionar_agrupamento', true);
        await this.modalAdicionar.show();
    }

    public fecharModal() {
        this.modalAdicionar.hide();
        this.modalAdicionar?.destroy();
    }

    public async abrirAdicaoDeContratos() {
        this.modalContratos = await ModalService.createFromJson('Faturamento/Modal/sm_next_00049_002_adicionar_contrato', true);
        await this.modalContratos.show();
    }

    public fecharAdicaoDeContrato() {
        this.modalContratos.hide();
        this.modalContratos?.destroy();
    }

    public setFiltroParceiro({ component }: any) {
        component.datasource.setFilter({
            CDTIPOPARCEIRO: ['PROSPECT', 'CLIENTE'],
        });
    }

    public dispararFiltro({ component }: IEventParam<TekGrid>) {
        const filterButton = component.getAtomInstance<Button>('FILTER_BUTTON');
        filterButton.click({} as Event, {} as HTMLElement);
    }

    public recarregarTabs() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabs-agrupamento');
        if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab-dadosGerais') {
            this.montarInformacoesDadosGerais();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab-contratos') {
            setTimeout(() => {
                const grid = Metadata.getInstance<TekGrid>('tek-grid-contratos');

                this.setFiltroGridContratos({ component: grid });
            }, 1000);
        }
    }

    public montarInformacoesDadosGerais() {
        const escopos = this.currentRow.NRTIPOORCAMENTO.split(',');
        const escoposArray: any = [];

        escopos.forEach((number: any) => {
            escoposArray.push(Number(number));
        });

        Metadata.getInstance<Text>('titulo-agrupamento').text = this.currentRow.NMAGRUPAMENTO;
        Metadata.getInstance<TextInput>('input-nomeAgrupamento-edit').value = this.currentRow.NMAGRUPAMENTO;
        Metadata.getInstance<Textarea>('textArea-observacoes-edit').value = this.currentRow.DSOBSAGRUPAMENTO;
        Metadata.getInstance<Select>('CDTIPOOPER').value = this.currentRow.CDTIPOOPERFATURA;
        Metadata.getInstance<Select>('NMTIPOOPER').value = this.currentRow.NMTIPOOPER;
        Metadata.getInstance<TextInput>('CDINSCESTA').value = this.currentRow.CDINSCESTA;
        Metadata.getInstance<TextInput>('CDINSCMUNI').value = this.currentRow.CDINSCMUNI;
        Metadata.getInstance<TextInput>('CDEMPRESA').value = this.currentRow.CDEMPRESA;
        Metadata.getInstance<Select>('CDSERIENF').value = this.currentRow.CDSERIENF;
        Metadata.getInstance<Select>('DSSERIE').value = this.currentRow.DSSERIE;
        Metadata.getInstance<Select>('CDFILIAL').value = this.currentRow.CDFILIAL;
        Metadata.getInstance<Select>('NMFILIAL').value = this.currentRow.NMFILIAL;
        Metadata.getInstance<SelectMultiple>('select-escopoFaturamento').setValue(escoposArray);
        Metadata.getInstance<Select>('NMPRINCIPALPARC').value = this.currentRow.NMPRINCIPALPARC;
        Metadata.getInstance<TextInput>('NRPARCFATURA').value = this.currentRow.NRPARCFATURA;
        Metadata.getInstance<TextInput>('NRAGRUPAMENTO').value = this.currentRow.NRAGRUPAMENTO;
        Metadata.getInstance<TextInput>('NMSECUNDARIPARC').value = this.currentRow.NMSECUNDARIPARC;
    }

    public setFiltroContrato({ component }: IEventParam<SelectMultiple>) {
        const { NRPARCFATURA } = this.currentRow;
        const { NRTIPOORCAMENTO } = this.currentRow;
        const { NRAGRUPAMENTO } = this.currentRow;

        component.datasource.setFilter({
            NRPARCFATURA,
            NRTIPOORCAMENTO,
            NRAGRUPAMENTO,
        });
    }

    public setFiltroGridContratos({ component }: any) {
        const nragrupamento = this.currentRow.NRAGRUPAMENTO;

        component.datasource.setFilter({
            NRAGRUPAMENTO: nragrupamento,
        });
    }

    public alertMessage(message: string, type: string) {
        AlertService.show({
            name: 'basic-alert',
            text: message,
            top: true,
            left: false,
            timeout: 3000,
            color: type,
        });
    }

    public dialogMessage(title: string, message: string) {
        DialogService.show({
            name: 'warning-dialog',
            type: 'warning',
            title,
            text: message,
        });
    }

    public validarEscoposAgrupamento(escopos: any) {
        return new Promise<void>((resolve, reject) => {
            if (escopos.length <= 1) {
                resolve();
            } else {
                LoadingService.show();
                Http.post('/validarEscoposAgrupamento', { NRTIPOORCAMENTO: escopos }).then((response: IDictionary) => {
                    if (response.data.dataset.agrupamento.IDESCSIMILARES) {
                        resolve();
                    } else {
                        DialogService.show({
                            name: 'buttons-dialog',
                            type: 'warning',
                            title: 'Atenção',
                            text: "Os escopos selecionados possuem parametrizações específicas para os serviços ligados a eles, podendo impossibilitar o agrupamento das parcelas com diferentes modelos de faturamento.<br>Ex: As parcelas de um faturamento 'Periódico' não serão unidas as parcelas de 'Parcelamentos'.<br><br>Deseja criar o agrupamento?",
                            buttons: [
                                {
                                    name: 'NAO',
                                    component: 'ZdButton',
                                    label: 'Não',
                                    outline: true,
                                    events: {
                                        click: () => {
                                            reject();
                                            DialogService.hide();
                                        },
                                    },
                                },
                                {
                                    name: 'SIM',
                                    component: 'ZdButton',
                                    label: 'Sim',
                                    events: {
                                        click: () => {
                                            resolve();
                                            LoadingService.show();
                                            DialogService.hide();
                                        },
                                    },
                                },
                            ],
                        });
                    }
                }).catch((error: any) => {
                    let mensagem = 'Falha ao validar os parâmetros de faturamento dos Escopos selecionados.';
                    if (error && error.error) {
                        if (error.error === 'PARAMS_DIFERENTE') {
                            mensagem = 'Operação bloqueada!<br><br>Não é possível criar agrupamentos com escopos que possuem diferentes parametrizações de faturamento.<br><br>Certifique-se que as informações de Tipo de Operação, Série da Nota Fiscal, Unidade para Faturamento e Tipo de Cálculo para Data de Saída são iguais';
                        } else if (error.error === 'COMPET_DIFERENTE') {
                            mensagem = 'Operação bloqueada!<br><br>Não é possível criar agrupamentos com escopos que possuem diferentes parametrizações de competência do faturamento.';
                        }
                    }
                    reject(mensagem);
                }).finally(() => {
                    LoadingService.hide();
                });
            }
        });
    }

    public salvarAgrupamento() {
        const form1 = Metadata.getInstance<Form>('form').validate();
        const form2 = Metadata.getInstance<Form>('form-informacoesParaAgrupamento-modal');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-agrupamento');

        if (form1 && form2.validate()) {
            const escopos = form2.value['select-escopoFaturamento'];

            this.validarEscoposAgrupamento(escopos).then(() => {
                LoadingService.show();
                return Http.post('/salvarAgrupamento', {
                    NRTIPOORCAMENTO: escopos,
                    NRPARCNEGOCIO: form2.value['select-parcnegocio'],
                    NMAGRUPAMENTO: Metadata.getInstance<TextInput>('input-nomeAgrupamento').value,
                    DSOBSAGRUPAMENTO: Metadata.getInstance<Textarea>('textArea-observacoes').value,
                });
            })
                .then(() => {
                    Utilities.showSuccessMessage();
                    this.fecharModal();
                    grid.datasource.get();
                })
                .catch((error: any) => {
                    Utilities.showErrorMessage(error);
                })
                .finally(() => {
                    LoadingService.hide();
                });
        } else {
            this.alertMessage('Preencha todos os campos obrigatórios (*)', 'error');
        }
    }

    public salvarEdicaoAgrupamento() {
        const form = Metadata.getInstance<Form>('form-editAgrupamento').validate();
        const grid = Metadata.getInstance<TekGrid>('tek-grid-agrupamento');
        const nm = Metadata.getInstance<TextInput>('input-nomeAgrupamento-edit').value;
        const ds = Metadata.getInstance<Textarea>('textArea-observacoes-edit').value;

        if (form) {
            LoadingService.show();
            Http.post('/updateAgrupamento', {
                NMAGRUPAMENTO: nm,
                DSOBSAGRUPAMENTO: ds,
                NRAGRUPAMENTO: Metadata.getInstance<TextInput>('NRAGRUPAMENTO').value,
            }).then(() => {
                Utilities.showSuccessMessage();
                this.currentRow.NMAGRUPAMENTO = nm;
                this.currentRow.DSOBSAGRUPAMENTO = ds;
                grid.datasource.get();
                this.montarInformacoesDadosGerais();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        } else {
            this.alertMessage('Preencha todos os campos obrigatórios (*)', 'error');
        }
    }

    public confirmarExclusaoAgrupamento() {
        DialogService.show({
            name: 'dialog-agrupamento',
            type: 'warning',
            title: 'Atenção',
            text: 'A exclusão do agrupamento resultará na regeração das parcelas de faturamento pendentes de seus contratos.<br><br>Deseja continuar?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    outline: true,
                    label: 'Não',
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deletarAgrupamento();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public async deletarAgrupamento() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-agrupamento');

        LoadingService.show();
        await Http.post('/deletarAgrupamento', {
            NRAGRUPAMENTO: Metadata.getInstance<TextInput>('NRAGRUPAMENTO').value,
        }).then(() => {
            Utilities.showSuccessMessage();
            grid.datasource.get();
            this.voltarPaginaInicial();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
        LoadingService.hide();
    }

    public confirmarExclusaoContratos({ event } : IEventParam<TekGrid>) {
        event?.preventDefault();
        DialogService.show({
            name: 'dialog-contratos',
            type: 'warning',
            title: 'Atenção',
            text: 'A exclusão de um contrato do agrupamento resultará no desmembramento das parcelas pendentes do faturamento em que ele participa.<br><br>Deseja continuar?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    outline: true,
                    label: 'Não',
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deletarContratos();
                            DialogService.hide();
                            LoadingService.show();
                        },
                    },
                },
            ],
        });
    }

    public deletarContratos() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-contratos');
        const NRS = (grid.selectedRows.map((element: any) => element.NRSEQCONTRATO)).toString();

        // não permite repetição de valores (new set)
        const nragrupamento = [
            ...new Set(
                grid.selectedRows.map((element: any) => element.NRAGRUPAMENTO),
            ),
        ].toString();

        Http.post('/deletarContratos', {
            NRAGRUPAMENTO: nragrupamento,
            NRSEQCONTRATO: NRS,
        }).then(() => {
            Utilities.showSuccessMessage();
            grid.selectedRows = [];
            grid.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public validarMoedaContratos(contratos: any) {
        const moeda = contratos.map((valor: any) => valor.NMMOEDA);
        const retorno = moeda.every((item:any) => item === moeda[0]);
        return retorno;
    }

    public validarContratos() {
        const formContratos = Metadata.getInstance<Form>('form-adicionarContrato').validate();
        const contratos = Metadata.getInstance<SelectMultiple>('select-contratos').selectValue;
        const NRSEQCONTRATO = Metadata.getInstance<SelectMultiple>('select-contratos').value;
        const NRAGRUPAMENTO = Metadata.getInstance<TextInput>('NRAGRUPAMENTO').value;
        let IDAPROVADOS: any;
        let contratosAprovados: any = [];
        let nrseq: any = [];

        if (formContratos) {
            if (this.validarMoedaContratos(contratos) === false) {
                const mensagemMoeda = 'Não é possível agrupar contratos com moedas diferentes, por favor selecione a mesma moeda.';
                this.dialogMessage('Operação bloqueada!', mensagemMoeda);
            } else {
                const contratosNaoAgrupam = contratos.filter((cont: any) => cont.IDAGRUPPARC !== '1').map((nameCont: any) => nameCont.NRCONTRATOFCT);
                if (contratosNaoAgrupam.length > 0) {
                    const mensagem = `<br/>Os seguintes contratos não podem ser agrupados pois estão parametrizados para gerar parcelas separadas por local de atendimento: <br/><br/>${contratosNaoAgrupam.join('<br/>')}`;
                    this.dialogMessage('Operação bloqueada!', mensagem);
                    // this.fecharAdicaoDeContrato();
                } else {
                    LoadingService.show();
                    Http.post('/validarContratosAprovados', {
                        NRSEQCONTRATO,
                    }).then((response: IDictionary) => {
                        LoadingService.hide();
                        IDAPROVADOS = (response.data.dataset.contratosAprovados.length === 0) ? 'N' : 'S';

                        if (IDAPROVADOS === 'S') {
                            contratosAprovados = response.data.dataset.contratosAprovados.map((nameCont: any) => nameCont.NRCONTRATOFCT);
                            nrseq = response.data.dataset.contratosAprovados.map((nrCont: any) => nrCont.NRSEQCONTRATO);

                            this.confirmarAdicaoContratosAprovados(contratosAprovados, nrseq, IDAPROVADOS, NRAGRUPAMENTO);
                        } else {
                            this.salvarContratos(NRSEQCONTRATO, IDAPROVADOS, NRAGRUPAMENTO);
                        }
                    }).catch((error: any) => {
                        LoadingService.hide();
                        Utilities.showErrorMessage(error);
                    });
                }
            }
        } else {
            this.alertMessage('Preencha todos os campos obrigatórios (*)', 'error');
        }
    }

    public confirmarAdicaoContratosAprovados(nrcontratofc: any, nrseq: any, IDAPROVADOS: string, NRAGRUPAMENTO: string) {
        LoadingService.hide();
        DialogService.show({
            name: 'dialog-contratos-aprovados',
            type: 'warning',
            title: 'Atenção!',
            text: `O agrupamento de contratos aprovados resultará na regeração do faturamento para agrupar suas parcelas de situação pendente. Contratos afetados: <br/><br/>${nrcontratofc.join('<br/>')}<br/><br/>Deseja prosseguir?`,
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            DialogService.hide();
                            this.salvarContratos(nrseq, IDAPROVADOS, NRAGRUPAMENTO);
                        },
                    },
                },
            ],
        });
    }

    public salvarContratos(NRSEQCONTRATO: any, IDAPROVADOS: string, NRAGRUPAMENTO: string) {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-contratos');

        LoadingService.show();
        Http.post('/salvarContratos', {
            NRSEQCONTRATO,
            NRAGRUPAMENTO,
            IDAPROVADOS,
        }).then(() => {
            Utilities.showSuccessMessage();
            this.fecharAdicaoDeContrato();
            grid.datasource.get();
        }).catch((error: any) => {
            if (error && error.error && error.error === 'PARC_DIFERENTES') {
                this.dialogMessage('Operação bloqueada!', 'Não é possível agrupar os contratos selecionados, porque possuem diferentes Parceiros para Faturamento.');
            } else {
                Utilities.showErrorMessage(error);
            }
        }).finally(() => {
            LoadingService.hide();
        });
    }
}
