import { Utilities } from '@/util/Utilities';
import {
    Button, DialogService, Form, LoadingService, Modal, ModalService, Select, Text, TextInput,
} from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { TekTreeGrid } from '@zeedhi/teknisa-components-common';

export class Sm00070000HierarquiaOperadoresController {
    private ModalHierarquia!: Modal;

    public currentRow: any = {};

    public dispararFiltroHierarquia({ component }: any) {
        const filterButton = Metadata.getInstance<Button>(`${component.name}_filterButton`);
        filterButton.click({} as Event, {} as HTMLElement);
    }

    public async abrirModalHierarquia() {
        await this.exibirModalHierarquia();
        Metadata.getInstance<TextInput>('formModalHierarquia-DSHIERARQUIAOPER').disabled = false;
        Metadata.getInstance<Text>('titleText').text = 'Nova Hierarquia';
    }

    public async exibirModalHierarquia() {
        this.ModalHierarquia = await ModalService.createFromJson('sm_00070_000_hierarquia_operadores/modal/sm_00070_000_hierarquia_operadores_modal', true);
        this.ModalHierarquia.show();
    }

    public fecharModalHierarquia() {
        this.ModalHierarquia.hide();
        this.ModalHierarquia?.destroy();
    }

    public async abrirModalEdicaoHierarquia({ row }: any) {
        await this.exibirModalHierarquia();
        this.currentRow = row;
        const operadorResponsavel = Metadata.getInstance<TextInput>('formModalHierarquia-DSHIERARQUIAOPER');
        operadorResponsavel.disabled = true;
        Metadata.getInstance<Text>('titleText').text = 'Editar Hierarquia';
        Metadata.getInstance<TextInput>('formModalHierarquia-NRHIERARQUIAOPER').value = row.NRHIERARQUIAOPER;
        operadorResponsavel.value = row.DSHIERARQUIAOPER;
        Metadata.getInstance<Select>('formModalHierarquia-NMFILIAL').value = row.CDFILIAL;
        Metadata.getInstance<Select>('formModalHierarquia-NRHIERARQUIAPAI').value = row.NRHIERARQUIAPAI;
        Metadata.getInstance<Select>('formModalHierarquia-IDOPERMASTER').value = row.IDOPERMASTER;
    }

    public insertHierarquia() {
        const form = Metadata.getInstance<Form>('formModalHierarquia');
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-hierarquia-operadores');
        if (Utilities.validateForm(form)) {
            LoadingService.show();
            let cdoperador;
            if (Metadata.getInstance<Select>('formModalHierarquia-NRHIERARQUIAPAI').value === null) {
                cdoperador = null;
            } else {
                cdoperador = Metadata.getInstance<Select>('formModalHierarquia-NRHIERARQUIAPAI').selectValue.CDOPERADOR;
            }
            Http.post('/insertHierarquia', {
                NRHIERARQUIAOPER: Metadata.getInstance<TextInput>('formModalHierarquia-NRHIERARQUIAOPER').value,
                DSHIERARQUIAOPER: Metadata.getInstance<Select>('formModalHierarquia-DSHIERARQUIAOPER').value,
                NRHIERARQUIAPAI: Metadata.getInstance<Select>('formModalHierarquia-NRHIERARQUIAPAI').value,
                CDOPERADOR: cdoperador,
                IDOPERMASTER: Metadata.getInstance<Select>('formModalHierarquia-IDOPERMASTER').value,
                CDFILIAL: Metadata.getInstance<Select>('formModalHierarquia-NMFILIAL').value,
            }).then(() => {
                this.fecharModalHierarquia();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showConfirmDeleteMessageHierarquia({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-hierarquia',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteHierarquia();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteHierarquia() {
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-hierarquia-operadores');
        LoadingService.show();
        Http.post('/deleteHierarquia', grid.selectedRows)
            .then(() => {
                grid.selectedRows = [];
                grid.reload();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
    }

    public controlaCamposSelecionados() {
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-hierarquia-operadores');
        grid.selectedRows.forEach((row) => {
            if (row.PAI === 'S') {
                grid.selectedRows = [];
            }
        });
    }
}
